@@ -3,30 +3,47 @@ Created on Tue Jan 25 16:34:41 2022
 
 @authors: Andrea Bassi @Polimi, Mark Neil @ImperialCollege
 """
-from napari_sim_processor.widget_settings import Setting
+from napari_sim_processor.widget_settings import Setting, Combo_box
 from napari_sim_processor.baseSimProcessor import pytorch, cupy
 from napari_sim_processor.hexSimProcessor import HexSimProcessor
 from napari_sim_processor.convSimProcessor import ConvSimProcessor
-from napari_sim_processor.simProcessor import SimProcessor
+# from napari_sim_processor.simProcessor import SimProcessor
 import napari
-from qtpy.QtWidgets import QVBoxLayout,QSplitter, QHBoxLayout, QWidget, QPushButton, QComboBox, QLineEdit
+from qtpy.QtWidgets import QVBoxLayout,QSplitter, QHBoxLayout, QWidget, QPushButton, QLineEdit
+from qtpy.QtCore import Qt
 from napari.layers import Image
 import numpy as np
 from napari.qt.threading import thread_worker
 from magicgui.widgets import FunctionGui
 from magicgui import magicgui, magic_factory
 import warnings
-import enum
 import time
 from superqt.utils import qthrottled
+from enum  import Enum
 
 
-class accel(enum.Enum):
-    USENUMPY = 1
-    USETORCH = 2
-    USECUPY = 3
+class Sim_modes(Enum):
+    SIM = 1
+    HEXSIM = 2
 
 
+class Accel(Enum):
+    USE_NUMPY = 1
+    USE_TORCH = 2
+    USE_CUPY = 3 
+    
+    @classmethod
+    def available(cls):
+        '''
+        Returns a new Enum with the available cpu/gpu accelerations 
+        '''
+        available_list = [cls.USE_NUMPY]
+        if pytorch: available_list.append(cls.USE_TORCH),
+        if cupy: available_list.append(cls.USE_CUPY) 
+        available_members = {member.name:member.value for member in available_list}
+        return Enum('AvailableAccel', available_members)
+    
+
 def reshape_init(reshape_widget: FunctionGui):
     @reshape_widget.input_image.changed.connect
     def _on_image_changed(input_image: Image):
@@ -103,7 +120,7 @@ def reshape(viewer: napari.Viewer,
             raise(ValueError('Input stack order reshaping not implemented'))
         input_image.data = rdata
         viewer.dims.axis_labels = ["angle", "phase", "z", "y","x"]
-        viewer.dims.set_point(axis=[0,1,2], value=[0,0,0]) #TODO  check why this is raising ValueError sistematically 
+        viewer.dims.set_point(axis=[0,1,2], value=[0,0,0]) #raises ValueError in napari versions <0.4.13 
 
 
 class SimAnalysis(QWidget):
@@ -127,14 +144,12 @@ class SimAnalysis(QWidget):
         self.start_sim_processor()
         self.viewer.dims.events.current_step.connect(self.on_step_change)
         
-    def setup_ui(self):     
         
+    def setup_ui(self):     
         def add_section(_layout):
-            from qtpy.QtCore import Qt
             splitter = QSplitter(Qt.Vertical)
             _layout.addWidget(splitter)
             #_layout.addWidget(QLabel(_title))
-            
         # initialize layout
         layout = QVBoxLayout()
         self.setLayout(layout)
@@ -147,8 +162,10 @@ class SimAnalysis(QWidget):
         right_layout = QVBoxLayout()
         bottom_layout.addLayout(right_layout)
         # Fill top layout
-        self.add_magic_layer_selection(top_layout)
+        self.add_magic_function(self.select_layer, top_layout)
         # Fill bottom-left layout
+        self.sim_mode = Combo_box(name = 'Mode', choices = Sim_modes, layout=left_layout,
+                                  write_function = self.reset_processor)
         self.phases_number = Setting('phases', dtype=int, initial=7, layout=left_layout, 
                               write_function = self.reset_processor)
         self.angles_number = Setting('angles', dtype=int, initial=1, layout=left_layout, 
@@ -190,27 +207,27 @@ class SimAnalysis(QWidget):
         # Fill bottom-right layout    
         self.carrier_idx = Setting('carrier index', dtype=int, initial=0,
                                    layout=right_layout, vmin = 0,
-                                   write_function = self.show_xcorr
+                                   write_function = self.show_functions
                                    )
         self.showXcorr = Setting('Show Xcorr', dtype=bool, initial=False,
                                      layout=right_layout,
-                                     write_function = self.show_xcorr
+                                     write_function = self.show_functions
                                      )
         self.showSpectrum = Setting('Show Spectrum', dtype=bool, initial=False,
                                      layout=right_layout,
-                                     write_function = self.show_spectrum
+                                     write_function = self.show_functions
                                      )
         self.showWiener = Setting('Show Wiener filter', dtype=bool, initial=False,
                                      layout=right_layout,
-                                     write_function = self.show_wiener
+                                     write_function = self.show_functions
                                      )
         self.showEta = Setting('Show Eta circle', dtype=bool, initial=False,
                                      layout=right_layout,
-                                     write_function = self.show_eta
+                                     write_function = self.show_functions
                                      )
         self.showCarrier = Setting('Show Carrier', dtype=bool, initial=False,
                                      layout=right_layout,
-                                     write_function = self.show_carrier
+                                     write_function = self.show_functions
                                      )
         self.keep_calibrating = Setting('Continuos Calibration', dtype=bool, initial=False,
                                      layout=right_layout, 
@@ -218,15 +235,11 @@ class SimAnalysis(QWidget):
         self.batch = Setting('Batch Reconstruction', dtype=bool, initial=False,
                                      layout=right_layout, 
                                      write_function = self.setReconstructor)
-
-        self.proc = QComboBox()
-        self.proc.addItem('Use Numpy', userData=accel.USENUMPY)
-        if pytorch:
-            self.proc.addItem('Use PyTorch', userData=accel.USETORCH)
-        if cupy:
-            self.proc.addItem('Use CuPy', userData=accel.USECUPY)
-        right_layout.addWidget(self.proc)
-
+        # creates the cpu/gpu acceleration combobox with only the availbale ones
+        self.proc = Combo_box(name = 'Accel', choices = Accel.available(),
+                                  layout=right_layout,
+                                  write_function = self.setReconstructor)    
+        # buttons
         buttons_dict = {'Widefield': self.calculate_WF_image,
                         'Calibrate': self.calibration,
                         'Plot calibration phases':self.find_phaseshifts,
@@ -238,16 +251,16 @@ class SimAnalysis(QWidget):
             button = QPushButton(button_name)
             button.clicked.connect(call_function)
             right_layout.addWidget(button)
-
         self.messageBox = QLineEdit()
         layout.addWidget(self.messageBox, stretch=True)
         self.messageBox.setText('Messages')
+    
         
-    def add_magic_layer_selection(self,_layout):
-        function = self.select_layer
+    def add_magic_function(self, function, _layout):
         self.viewer.layers.events.inserted.connect(function.reset_choices)
         self.viewer.layers.events.removed.connect(function.reset_choices)
         _layout.addWidget(function.native)
+        
     
     @magicgui(call_button='Select image layer')    
     def select_layer(self, image: Image):
@@ -276,15 +289,23 @@ class SimAnalysis(QWidget):
         self.viewer.dims.axis_labels = ["angle", "phase", "z", "y","x"]
         self.rescaleZ()
         self.center_stack(image)
-        self.start_sim_processor()
         self.move_layer_to_top(image)
+        self.reset_processor()
+        print(f'Selected image layer: {image.name}')
+    
+    
+    def reset_processor(self,*args):
+        '''
+        Reset the processor and starts it (stops it if existing). 
+        Disables xcorr,spectrum and circle.
+        ''' 
+        self.start_sim_processor()
         self.showXcorr.val = False
+        self.showWiener.val = False
         self.showSpectrum.val = False
         self.showEta.val = False
         self.showCarrier.val = False
         self.keep_calibrating.val = False
-        #self.keep_reconstructing.val = False
-        print(f'Selected image layer: {image.name}')
         
             
     def rescaleZ(self):
@@ -311,19 +332,15 @@ class SimAnalysis(QWidget):
                 current_step[dim_idx] = data.shape[dim_idx]//2
             self.viewer.dims.current_step = current_step                
            
+            
     @qthrottled (timeout=10)
     def on_step_change(self, *args):   
         if hasattr(self, 'imageRaw_name'):
             t0 = time.time()
             self.setReconstructor()
-            self.show_spectrum()
-            self.show_xcorr()
             min_timeout = 10 #ms
             delta_t = time.time() - t0
-            self.set_timeout(delta_t + min_timeout)
-
-    def set_timeout(self, new_timeout):
-        self.on_step_change.set_timeout(new_timeout)
+            self.on_step_change.set_timeout(delta_t + min_timeout)
 
 
     def show_image(self, image_values, im_name, **kwargs):
@@ -350,7 +367,7 @@ class SimAnalysis(QWidget):
                                             colormap = colormap,
                                             interpolation = 'bilinear')
         self.center_stack(image_values)
-        self.move_layer_to_top(layer)
+        # self.move_layer_to_top(layer)
         if kwargs.get('autoscale') is True:
             layer.reset_contrast_limits()
         return layer
@@ -416,6 +433,7 @@ class SimAnalysis(QWidget):
         stack = fullstack[:,:,z_index,:,:]
         return stack
     
+    
     def get_current_stack_for_calibration(self):
         '''
         Returns the 4D raw image (angles,phases,y,x) stack at the z value selected in the viewer  
@@ -451,41 +469,34 @@ class SimAnalysis(QWidget):
         ''''
         Creates an instance of the Processor
         '''
-        self.isCalibrated = False
-        if hasattr(self, 'h'):
-            self.stop_sim_processor()
-            self.start_sim_processor()
-        else:
-            if self.phases_number.val == 3 and self.angles_number.val == 1: 
-                self.h = SimProcessor()  
-                k_shape = (1,1)
-            elif self.phases_number.val == 7:  
-                self.h = HexSimProcessor()  
-                k_shape = (3,1)
-            elif self.phases_number.val >= 3 and self.angles_number.val > 0:
-                self.h = ConvSimProcessor(angleSteps=self.angles_number.val,
-                                          phaseSteps=self.phases_number.val)
-                k_shape = (self.angles_number.val,1)
-            else: 
-                raise(ValueError("Invalid phases or angles number"))
-            self.h.debug = False
-            self.setReconstructor() 
-            self.kx_input = np.zeros(k_shape, dtype=np.single)
-            self.ky_input = np.zeros(k_shape, dtype=np.single)
-            self.p_input = np.zeros(k_shape, dtype=np.single)
-            self.ampl_input = np.zeros(k_shape, dtype=np.single)
+        if self.is_image_in_layers():
+            self.isCalibrated = False
+            if hasattr(self, 'h'):
+                self.stop_sim_processor()
+                self.start_sim_processor()
+            else:
+                if self.sim_mode.current_data == Sim_modes.HEXSIM.value:  
+                    self.h = HexSimProcessor()  
+                    k_shape = (3,1)
+                elif self.sim_mode.current_data == Sim_modes.SIM.value and self.phases_number.val >= 3 and self.angles_number.val > 0:
+                    self.h = ConvSimProcessor(angleSteps=self.angles_number.val,
+                                              phaseSteps=self.phases_number.val)
+                    k_shape = (self.angles_number.val,1)   
+                else: 
+                    raise(ValueError("Invalid phases or angles number"))
+                self.carrier_idx.set_min_max(0,k_shape[0]-1) # TODO connect carrier idx to angle if Sim_mode == SIM
+                self.h.debug = False
+                self.setReconstructor() 
+                self.kx_input = np.zeros(k_shape, dtype=np.single)
+                self.ky_input = np.zeros(k_shape, dtype=np.single)
+                self.p_input = np.zeros(k_shape, dtype=np.single)
+                self.ampl_input = np.zeros(k_shape, dtype=np.single)
 
             
     def stop_sim_processor(self):
         if hasattr(self, 'h'):
             delattr(self, 'h')
-  
-    
-    def reset_processor(self,*args):
-        self.isCalibrated = False
-        self.stop_sim_processor()
-        self.start_sim_processor()
-       
+
     
     def setReconstructor(self,*args):
         '''
@@ -508,12 +519,9 @@ class SimAnalysis(QWidget):
                 self.h.ky = self.ky_input
             if self.keep_calibrating.val:
                 self.calibration()
-            # if self.keep_reconstructing.val:
-            #     self.single_plane_reconstruction()
-            if self.showEta.val:
-                self.show_eta()
+            self.show_functions()
           
-            
+           
     def show_wiener(self, *args):
         """
         Shows the Wiener filter 
@@ -526,12 +534,10 @@ class SimAnalysis(QWidget):
                 swy,swx = img.shape
                 self.show_image(img[swy//2-swy//4:swy//2+swy//4,swx//2-swx//4:swx//2+swx//4],
                                 imname, hold = True, scale=[1,1])
-                self.show_carrier()
-                self.show_eta()
             elif not self.showWiener.val and imname in self.viewer.layers:
                 self.remove_layer(self.viewer.layers[imname])
                        
-            
+       
     def show_spectrum(self, *args):
         """
         Calculates and shows the power spectrum of the image
@@ -544,12 +550,10 @@ class SimAnalysis(QWidget):
                 epsilon = 1e-10
                 ps = np.log((np.abs(fftshift(fft2(img0))))**2+epsilon)
                 self.show_image(ps, imname, hold = True)
-                self.show_carrier()
-                self.show_eta()
             elif not self.showSpectrum.val and imname in self.viewer.layers:
                 self.remove_layer(self.viewer.layers[imname])
        
-     
+    
     def show_xcorr(self, *args):
         """
         Show the crosscorrelation of the low and high pass filtered version of the raw images,
@@ -560,27 +564,30 @@ class SimAnalysis(QWidget):
             if self.showXcorr.val and hasattr(self,'h'):
                 im = self.get_current_stack_for_calibration()
                 # choose the gpu acceleration
-                if self.proc.currentData() == accel.USETORCH:
-                    ixf = np.squeeze(self.h.crossCorrelations_pytorch(im))
-                elif self.proc.currentData() == accel.USECUPY:
+                if self.proc.current_data == Accel.USE_TORCH.value:
+                    ixf = np.squeeze(self.h.crossCorrelations_pytorch(im)) #TODO check if traspose or flip should be added
+                elif self.proc.current_data == Accel.USE_CUPY.value:
                     ixf = np.squeeze(self.h.crossCorrelations_cupy(im))
                 else:
                     ixf = np.squeeze(self.h.crossCorrelations(im))
-                # show the slected carrier
+                # show the selected carrier
                 if ixf.ndim >2:
-                    carriers_number = ixf.shape[0]
-                    self.carrier_idx.set_min_max(0,carriers_number-1)
                     carrier_idx = self.carrier_idx.val
                     ixf =ixf [carrier_idx,:,:]   
                 else:
                     self.carrier_idx.set_min_max(0,0)
                 self.show_image(ixf, imname, hold = True,
                                 colormap ='twilight', autoscale = True)
-                self.show_carrier()
-                self.show_eta()
             elif not self.showXcorr.val and imname in self.viewer.layers:
                 self.remove_layer(self.viewer.layers[imname])
-            
+    
+    def show_functions(self, *args):
+        self.show_wiener()
+        self.show_spectrum()
+        self.show_xcorr()
+        self.show_carrier()
+        self.show_eta()
+        
      
     def calculate_kr(self,N):  
         '''
@@ -598,30 +605,32 @@ class SimAnalysis(QWidget):
         cutoff_in_pixels = cutoff / dk
         return cutoff_in_pixels, dk   
       
-      
+     
     def show_carrier(self, *args):
         '''
         Draws the carrier frenquencies in a shape layer
         '''
-        if self.is_image_in_layers() and self.isCalibrated:
+        if self.is_image_in_layers() :
             name = f'carrier_{self.imageRaw_name}'
-            if self.showCarrier.val:
+            if self.showCarrier.val and self.isCalibrated:
                 N = self.h.N
                 cutoff, dk = self.calculate_kr(N)
                 kxs = self.h.kx
                 kys = self.h.ky
                 pc = np.zeros((len(kxs),2))
+                radii = []
                 for idx, (kx,ky) in enumerate(zip(kxs,kys)):
                     pc[idx,0] = ky[0] / dk + N/2
                     pc[idx,1] = kx[0] / dk + N/2
-                radius = self.h.N // 30 # radius of the displayed circle 
-                self.add_circles(pc, radius, name, color='red')
+                    radii.append(self.h.N // 30) # radius of the displayed circle
+                layer=self.add_circles(pc, radii, name, color='red')
+                self.move_layer_to_top(layer) 
                 # kr = np.sqrt(kxs**2+kys**2)
                 # print('Carrier magnitude / cut off:', *kr/cutoff*dk)
             elif name in self.viewer.layers:
                 self.remove_layer(self.viewer.layers[name])
-                
-           
+    
+                   
     def show_eta(self):
         '''
         Shows two circles with radius eta (green circle), 
@@ -629,20 +638,27 @@ class SimAnalysis(QWidget):
         '''
         if self.is_image_in_layers():
             name = f'eta_circle_{self.imageRaw_name}'
+            centres = []
+            radii = []
+            colors = []
             if self.showEta.val:
                 N = self.h.N
                 cutoff, dk   = self.calculate_kr(N)  
                 eta_radius = 1.9 * self.h.eta * cutoff
-                self.add_circles(np.array([N/2,N/2]), eta_radius,
-                               name, color='green')
-                self.add_circles(np.array([N/2,N/2]), 2 * cutoff,
-                               name, color='blue', hold=True)
+                centres.append(np.array([N/2,N/2]))
+                radii.append(eta_radius)
+                colors.append('green')
+                centres.append(np.array([N/2,N/2]))
+                radii.append(2 * cutoff)
+                colors.append('blue')
+                layer=self.add_circles(centres, radii, shape_name=name, color=colors)
+                self.move_layer_to_top(layer)
             elif name in self.viewer.layers:
-                self.remove_layer(self.viewer.layers[name])
+                self.remove_layer(self.viewer.layers[name])   
 
-    def add_circles(self, locations, radius=20,
-                    shape_name='shapename', color='blue', hold=False
-                    ):
+    
+    def add_circles(self, locations, radii,
+                    shape_name='shapename', color='blue', hold=False):
         '''
         Creates a circle in a layer with yx coordinates speciefied in each row of locations
         
@@ -652,20 +668,20 @@ class SimAnalysis(QWidget):
             yx coordinates of the centers. 
         shape_name : str
             name of the new Shape
-        radius : int 
-            radius of the circles.
-        color : str of RGBA list
+        radii : int
+            radii of the circles.
+        color : list of or single instance of str or RGBA list
             color of the circles.
         hold : bool
             if True updates the existing layer, with name shape_name,
             without creating a new layer
         '''
         ellipses = []
-        for center in locations: 
-            bbox = np.array([center+np.array([radius, radius]),
-                             center+np.array([radius,-radius]),
-                             center+np.array([-radius,-radius]),
-                             center+np.array([-radius, radius])]
+        for center, radius in zip(locations, radii):
+            bbox = np.array([center + np.array([radius, radius]),
+                             center + np.array([radius, -radius]),
+                             center + np.array([-radius, -radius]),
+                             center + np.array([-radius, radius])]
                             )
             ellipses.append(bbox)
         
@@ -674,14 +690,13 @@ class SimAnalysis(QWidget):
             if hold:
                 circles_layer.add_ellipses(ellipses, edge_color=color)
             else:
-                circles_layer.data = np.array(ellipses)
+                circles_layer.data = ellipses
         else:  
             circles_layer = self.viewer.add_shapes(name=shape_name,
                                    edge_width = 1.3,
-                                   face_color = [1,1,1,0],
-                                   edge_color = color)
+                                   face_color = [1, 1, 1, 0])
             circles_layer.add_ellipses(ellipses, edge_color=color)
-        self.move_layer_to_top(circles_layer)   
+        return circles_layer
     
     
     def showCalibrationTable(self):
@@ -705,9 +720,9 @@ class SimAnalysis(QWidget):
         phases_angles = sa*sp
         pa_stack = fullstack.reshape(phases_angles, sz, sy, sx)
         paz_stack = np.swapaxes(pa_stack, 0, 1).reshape((phases_angles*sz, sy, sx))
-        if self.proc.currentData() == accel.USETORCH:
+        if self.proc.current_data == Accel.USE_TORCH.value:
             demodulation_function = self.h.filteredOSreconstruct_pytorch
-        elif self.proc.currentData() == accel.USECUPY:
+        elif self.proc.current_data == Accel.USE_CUPY.value:
             demodulation_function = self.h.filteredOSreconstruct_cupy
         else:
             demodulation_function = self.h.filteredOSreconstruct
@@ -742,9 +757,9 @@ class SimAnalysis(QWidget):
         if hasattr(self, 'h'):
             imRaw = self.get_current_stack_for_calibration()
             start_time = time.time()
-            if self.proc.currentData() == accel.USETORCH:
+            if self.proc.current_data == Accel.USE_TORCH.value:
                 self.h.calibrate_pytorch(imRaw,self.find_carrier.val)
-            elif self.proc.currentData() == accel.USECUPY:
+            elif self.proc.current_data == Accel.USE_CUPY.value:
                 self.h.calibrate_cupy(imRaw, self.find_carrier.val)
             else:
                 self.h.calibrate(imRaw,self.find_carrier.val)
@@ -756,10 +771,7 @@ class SimAnalysis(QWidget):
                 self.ky_input = self.h.ky
                 self.p_input = self.h.p
                 self.ampl_input = self.h.ampl 
-            self.show_wiener()
-            self.show_xcorr()
-            self.show_carrier()
-            self.show_eta()
+            self.show_functions()
             
                    
     def single_plane_reconstruction(self):
@@ -771,9 +783,9 @@ class SimAnalysis(QWidget):
         dshape= current_image.shape
         phases_angles = self.phases_number.val*self.angles_number.val
         rdata = current_image.reshape(phases_angles, dshape[-2],dshape[-1])
-        if self.proc.currentData() == accel.USETORCH:
+        if self.proc.current_data == Accel.USE_TORCH.value:
             imageSIM = self.h.reconstruct_pytorch(rdata.astype(np.float32)) #TODO:this is left after conversion from torch
-        elif self.proc.currentData() == accel.USECUPY:
+        elif self.proc.current_data == Accel.USE_CUPY.value:
             imageSIM = self.h.reconstruct_cupy(rdata.astype(np.float32))  # TODO:this is left after conversion from torch
         else:
             imageSIM = self.h.reconstruct_rfftw(rdata)
@@ -809,15 +821,15 @@ class SimAnalysis(QWidget):
                         data = pa_stack[:,zmin:zmax,:,:]
                         s_pa = data.shape[0]
                         selected_imRaw = np.swapaxes(data, 0, 1).reshape((s_pa * new_delta, sy, sx))
-                        if self.proc.currentData() == accel.USETORCH:
+                        if self.proc.current_data == Accel.USE_TORCH.value:
                             self.h.calibrate_pytorch(selected_imRaw,self.find_carrier.val)
-                        elif self.proc.currentData() == accel.USECUPY:
+                        elif self.proc.current_data == Accel.USE_CUPY.value:
                             self.h.calibrate_cupy(selected_imRaw, self.find_carrier.val)
                         else:
                             self.h.calibrate(selected_imRaw,self.find_carrier.val)                
-                if self.proc.currentData() == accel.USETORCH:
+                if self.proc.current_data == Accel.USE_TORCH.value:
                     stackSIM[zidx,:,:] = self.h.reconstruct_pytorch(phases_stack.astype(np.float32)) #TODO:this is left after conversion from torch
-                elif self.proc.currentData() == accel.USECUPY:
+                elif self.proc.current_data == Accel.USE_CUPY.value:
                     stackSIM[zidx, :, :] = self.h.reconstruct_cupy(phases_stack.astype(np.float32))  # TODO:this is left after conversion from torch
                 else:
                     stackSIM[zidx,:,:] = self.h.reconstruct_rfftw(phases_stack)      
@@ -827,9 +839,9 @@ class SimAnalysis(QWidget):
         def _batch_reconstruction():
             warnings.filterwarnings('ignore')
             start_time = time.time()
-            if self.proc.currentData() == accel.USETORCH:
+            if self.proc.current_data == Accel.USE_TORCH.value:
                 stackSIM = self.h.batchreconstructcompact_pytorch(paz_stack, blocksize = 32)
-            elif self.proc.currentData() == accel.USECUPY:
+            elif self.proc.current_data == Accel.USE_CUPY.value:
                 stackSIM = self.h.batchreconstructcompact_cupy(paz_stack, blocksize=32)
             else:
                 stackSIM = self.h.batchreconstructcompact(paz_stack)
@@ -852,9 +864,9 @@ class SimAnalysis(QWidget):
           
     def find_phaseshifts(self):
         assert self.isCalibrated, 'SIM processor not calibrated, unable to show phases'
-        if self.phases_number.val==7:
+        if self.sim_mode.current_data == Sim_modes.HEXSIM.value:
             self.find_hexsim_phaseshifts()
-        elif self.phases_number.val==3 or self.phases_number.val==5 : #TODO make more general with combobox
+        elif self.sim_mode.current_data == Sim_modes.SIM.value:
             self.find_sim_phaseshifts()
         
         
@@ -866,9 +878,9 @@ class SimAnalysis(QWidget):
         sa,sp,sy,sx = stack.shape
         img = stack.reshape(sa*sp, sy, sx) 
         for i in range (3):
-            if self.proc.currentData() == accel.USETORCH:
+            if self.proc.current_data == Accel.USE_TORCH.value:
                 phase, _ = self.h.find_phase_pytorch(self.h.kx[i], self.h.ky[i], img)
-            elif self.proc.currentData() == accel.USECUPY:
+            elif self.proc.current_data == Accel.USE_CUPY.value:
                 phase, _ = self.h.find_phase_cupy(self.h.kx[i], self.h.ky[i], img)
             else:
                 phase, _ = self.h.find_phase(self.h.kx[i], self.h.ky[i], img)
@@ -890,9 +902,9 @@ class SimAnalysis(QWidget):
             phaseshift = np.zeros((sp,sa))
             expected_phase = np.zeros((sp,sa))
             error = np.zeros((sp,sa))
-            if self.proc.currentData() == accel.USETORCH:
+            if self.proc.current_data == Accel.USE_TORCH.value:
                 phase, _ = self.h.find_phase_pytorch(self.h.kx[angle_idx], self.h.ky[angle_idx], img)
-            elif self.proc.currentData() == accel.USECUPY:
+            elif self.proc.current_data == Accel.USE_CUPY.value:
                 phase, _ = self.h.find_phase_cupy(self.h.kx[angle_idx], self.h.ky[angle_idx], img)
             else:
                 phase, _ = self.h.find_phase(self.h.kx[angle_idx], self.h.ky[angle_idx], img)
